from ...utils.containers import ClusterConductor
from ...utils.testcase import TestCase
from ...utils.util import Logger
from ..helper import KVSMultiClient, KVSTestFixture
from ...utils.kvs_api import DEFAULT_TIMEOUT

def basic_kv_view_accept(conductor: ClusterConductor, dir, log: Logger):
    with KVSTestFixture(conductor, dir, log, node_count=4) as fx:
        c = KVSMultiClient(fx.clients, "client", log)
        conductor.add_shard("shard1", conductor.get_nodes([0, 1, 2]))
        fx.broadcast_view(conductor.get_shard_view())

        r = c.put(0, "x", "1")
        assert r.ok, f"expected ok for new key, got {r.status_code}"

        r = c.put(1, "y", "2")
        assert r.ok, f"expected ok for new key, got {r.status_code}"

        r = c.get(0, "x")
        assert r.ok, f"expected ok for get, got {r.status_code}"
        assert r.json()["value"] == "1", f"wrong value returned: {r.json()}"

        r = c.get(1, "x")
        assert r.ok, f"expected ok for get, got {r.status_code}"
        assert r.json()["value"] == "1", f"wrong value returned: {r.json()}"

        r = c.get(0, "y")
        assert r.ok, f"expected ok for get, got {r.status_code}"
        assert r.json()["value"] == "2", f"wrong value returned: {r.json()}"

        r = c.get(1, "y")
        assert r.ok, f"expected ok for get, got {r.status_code}"
        assert r.json()["value"] == "2", f"wrong value returned: {r.json()}"

        return True, 0


def basic_kv_1(conductor: ClusterConductor, dir, log: Logger):
    with KVSTestFixture(conductor, dir, log, node_count=4) as fx:
        c = KVSMultiClient(fx.clients, "client", log)
        conductor.add_shard("shard1", conductor.get_nodes([0]))
        conductor.add_shard("shard2", conductor.get_nodes([1]))
        fx.broadcast_view(conductor.get_shard_view())

        r = c.put(0, "x", "1")
        assert r.ok, f"expected ok for new key, got {r.status_code}"

        r = c.put(1, "y", "2")
        assert r.ok, f"expected ok for new key, got {r.status_code}"

        r = c.get(0, "x")
        assert r.ok, f"expected ok for get, got {r.status_code}"
        assert r.json()["value"] == "1", f"wrong value returned: {r.json()}"

        r = c.get(1, "x")
        assert r.ok, f"expected ok for get, got {r.status_code}"
        assert r.json()["value"] == "1", f"wrong value returned: {r.json()}"

        r = c.get(0, "y")
        assert r.ok, f"expected ok for get, got {r.status_code}"
        assert r.json()["value"] == "2", f"wrong value returned: {r.json()}"

        r = c.get(1, "y")
        assert r.ok, f"expected ok for get, got {r.status_code}"
        assert r.json()["value"] == "2", f"wrong value returned: {r.json()}"

        return True, 0


BASIC_TESTS = [TestCase("basic_kv_1", basic_kv_1), TestCase("basic_kv_view_accept", basic_kv_view_accept)]
